@@ -17,8 +17,8 @@ Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this program; if not, write to the
-Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.
 
 Derived from eel-background.c and eel-gdk-pixbuf-extensions.c by
 Darin Adler <darin@eazel.com> and Ramiro Estrugo <ramiro@eazel.com>
@@ -84,6 +84,7 @@ struct _FileSize
  * bad dithering when tiling the gradient
  */
 #define GRADIENT_PIXMAP_TILE_SIZE 128
+#define THUMBNAIL_SIZE 256
 
 typedef struct FileCacheEntry FileCacheEntry;
 #define CACHE_SIZE 4
@@ -290,6 +291,39 @@ queue_transitioned (GnomeBG *bg)
 					     NULL);
 }
 
+static gboolean 
+bg_gsettings_mapping (GVariant *value,
+			gpointer *result,
+			gpointer user_data)
+{
+	const gchar *bg_key_value;
+	char *filename = NULL;
+
+	/* The final fallback if nothing matches is with a NULL value. */
+	if (value == NULL) {
+		*result = NULL;
+		return TRUE;
+	}
+
+	bg_key_value = g_variant_get_string (value, NULL);
+
+	if (bg_key_value && *bg_key_value != '\0') {
+		filename = g_filename_from_uri (bg_key_value, NULL, NULL);
+
+		if (filename != NULL && g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE) {
+			g_free (filename);
+			return FALSE;
+		}
+
+		if (filename != NULL) {
+			*result = filename;
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 void
 gnome_bg_load_from_preferences (GnomeBG   *bg,
 				GSettings *settings)
@@ -306,16 +340,7 @@ gnome_bg_load_from_preferences (GnomeBG   *bg,
 	bg->is_enabled = g_settings_get_boolean (settings, BG_KEY_DRAW_BACKGROUND);
 
 	/* Filename */
-	filename = NULL;
-	tmp = g_settings_get_string (settings, BG_KEY_PICTURE_URI);
-	if (tmp && *tmp != '\0') {
-		filename = g_filename_from_uri (tmp, NULL, NULL);
-		if (filename != NULL && g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE) {
-			g_free (filename);
-			filename = NULL;
-		}
-	}
-	g_free (tmp);
+	filename = g_settings_get_mapped (settings, BG_KEY_PICTURE_URI, bg_gsettings_mapping, NULL);
 
 	/* Colors */
 	tmp = g_settings_get_string (settings, BG_KEY_PRIMARY_COLOR);
@@ -1056,19 +1081,21 @@ gnome_bg_get_pixmap_size (GnomeBG   *bg,
  * @window: 
  * @width: 
  * @height:
- * @is_root:
+ * @root:
  *
  * Create a surface that can be set as background for @window. If @is_root is
  * TRUE, the surface created will be created by a temporary X server connection
  * so that if someone calls XKillClient on it, it won't affect the application
  * who created it.
+ *
+ * Returns: %NULL on error (e.g. out of X connections)
  **/
 cairo_surface_t *
 gnome_bg_create_surface (GnomeBG	    *bg,
 		 	 GdkWindow   *window,
 			 int	     width,
 			 int	     height,
-			 gboolean     is_root)
+			 gboolean     root)
 {
 	int pm_width, pm_height;
 	cairo_surface_t *surface;
@@ -1088,7 +1115,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 	/* has the side effect of loading and caching pixbuf only when in tile mode */
 	gnome_bg_get_pixmap_size (bg, width, height, &pm_width, &pm_height);
 	
-	if (is_root) {
+	if (root) {
 		surface = make_root_pixmap (gdk_window_get_screen (window),
 					   pm_width, pm_height);
 	}
@@ -1097,13 +1124,16 @@ gnome_bg_create_surface (GnomeBG	    *bg,
                                                              CAIRO_CONTENT_COLOR,
                                                              pm_width, pm_height);
 	}
-	
+
+	if (surface == NULL)
+		return NULL;
+
 	cr = cairo_create (surface);
 	if (!bg->filename && bg->color_type == G_DESKTOP_BACKGROUND_SHADING_SOLID) {
 		gdk_cairo_set_source_color (cr, &(bg->primary));
-		average.red = bg->primary.red / 255.0;
-		average.green = bg->primary.green / 255.0;
-		average.blue = bg->primary.blue / 255.0;
+		average.red = bg->primary.red / 65535.0;
+		average.green = bg->primary.green / 65535.0;
+		average.blue = bg->primary.blue / 65535.0;
 		average.alpha = 1.0;
 	}
 	else {
@@ -1111,7 +1141,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 		
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
 					 width, height);
-		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (window), is_root);
+		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (window), root);
 		gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
 		pixbuf_average_value (pixbuf, &average);
 		g_object_unref (pixbuf);
@@ -1153,12 +1183,13 @@ gnome_bg_is_dark (GnomeBG *bg,
 	pixbuf = get_pixbuf_for_size (bg, -1, width, height);
 	if (pixbuf) {
 		GdkRGBA argb;
+		guchar a, r, g, b;
 
 		pixbuf_average_value (pixbuf, &argb);
-		guchar a = argb.alpha * 0xff;
-		guchar r = argb.red * 0xff;
-		guchar g = argb.green * 0xff;
-		guchar b = argb.blue * 0xff;
+		a = argb.alpha * 0xff;
+		r = argb.red * 0xff;
+		g = argb.green * 0xff;
+		b = argb.blue * 0xff;
 		
 		color.red = (color.red * (0xFF - a) + r * 0x101 * a) / 0xFF;
 		color.green = (color.green * (0xFF - a) + g * 0x101 * a) / 0xFF;
@@ -1534,6 +1565,7 @@ gnome_bg_set_surface_as_root (GdkScreen *screen, cairo_surface_t *surface)
 	int      screen_num;
 
 	g_return_if_fail (screen != NULL);
+	g_return_if_fail (surface != NULL);
 	g_return_if_fail (cairo_surface_get_type (surface) == CAIRO_SURFACE_TYPE_XLIB);
 
 	screen_num = gdk_screen_get_number (screen);
@@ -2599,7 +2631,6 @@ pixbuf_tile (GdkPixbuf *src, GdkPixbuf *dest)
 	int tile_width, tile_height;
 	int dest_width = gdk_pixbuf_get_width (dest);
 	int dest_height = gdk_pixbuf_get_height (dest);
-	
 	tile_width = gdk_pixbuf_get_width (src);
 	tile_height = gdk_pixbuf_get_height (src);
 	
@@ -2953,7 +2984,7 @@ read_slideshow_file (const char *filename,
 		/* one slide, there's no transition */
 		} else if (len == 1) {
 			Slide *slide = show->slides->head->data;
-			slide->duration = G_MAXUINT;
+			slide->duration = show->total_duration = G_MAXUINT;
 		}
 	}
 
@@ -2994,7 +3025,7 @@ create_thumbnail_for_filename (GnomeDesktopThumbnailFactory *factory,
 			int orig_width = gdk_pixbuf_get_width (orig);
 			int orig_height = gdk_pixbuf_get_height (orig);
 			
-			result = pixbuf_scale_to_fit (orig, 128, 128);
+			result = pixbuf_scale_to_fit (orig, THUMBNAIL_SIZE, THUMBNAIL_SIZE);
 			
 			g_object_set_data_full (G_OBJECT (result), "gnome-thumbnail-height",
 						g_strdup_printf ("%d", orig_height), g_free);
